# Generated by Django 2.0.4 on 2018-12-13 12:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0008_pre_switch_namespace_field'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='translationkey',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='translationkey',
            name='namespace',
        ),
    ]



# Generated by Django 2.0.4 on 2018-05-16 18:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0002_auto_20180516_0958'),
    ]

    operations = [
        migrations.AddField(
            model_name='translationkey',
            name='namespace',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='namespace'),
        ),
        migrations.AddField(
            model_name='translationkey',
            name='public',
            field=models.BooleanField(default=True),
        ),
    ]



# Generated by Django 2.0.4 on 2018-12-13 12:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0009_auto_20181213_1239'),
    ]

    operations = [
        migrations.RenameField(
            model_name='translationkey',
            old_name='new_namespace',
            new_name='namespace',
        ),
    ]



import polib
from django.core.management.base import BaseCommand
from tradukoj.models import Translation, TranslationKey


class Command(BaseCommand):
    # pylint: disable=C0301
    help = 'Generate a .po file of langtag'

    def add_arguments(self, parser):

        parser.add_argument(
            '--langtag',
            dest='langtag',
            required=True,
            help='tag used in BCP47',
        )

        parser.add_argument(
            '--output',
            dest='output',
            required=True,
            help='output file',
        )

        parser.add_argument(
            '--namespace',
            dest='namespace',
            required=True,
            help='namespace of translations',
        )

        parser.add_argument(
            '--reference_langtag',
            dest='reference_langtag',
            required=True,
            help='translated text used in comments to help translator team',
        )

    def handle(self, *args, **options):
        langtag = options['langtag']
        output_file = options['output']
        reference_langtag = options['reference_langtag']
        namespace = options['namespace']

        po_object = polib.POFile()
        # TODO: allow configure metadata
        po_object.metadata = {
            'Content-Type': 'text/plain; charset=utf-8',
        }
        for translation_key in TranslationKey.objects.filter(
                namespace__text=namespace):

            translated = ""
            try:
                translation = Translation.objects.get(
                    bcp47__langtag=langtag, key=translation_key)
                translated = translation.str_translation()
            except Translation.DoesNotExist:
                translated = ""

            if not translation_key.text:
                continue

            help_text = ""
            try:
                reference_translation = Translation.objects.get(
                    bcp47__langtag=reference_langtag, key=translation_key)
                help_text = f"i18n {reference_langtag}: {reference_translation.str_translation()}"
            except Translation.DoesNotExist:
                help_text = "i18n {reference_langtag}: "

            if not translated:
                translated = ""
            entry = polib.POEntry(
                msgid=translation_key.text,
                msgstr=translated,
                comment=help_text)
            po_object.append(entry)

        po_object.save(output_file)



from langcodes import best_match
from django.utils.translation.trans_real import parse_accept_lang_header
from rest_framework import serializers
from .models import Translation, BCP47, TranslationKey


class BCP47Serializer(serializers.ModelSerializer):
    best_match_score = serializers.SerializerMethodField()

    class Meta:
        model = BCP47
        fields = ('langtag', 'name', 'best_match_score', 'default', 'direction')

    # TODO: Implement MaxMind Geo-IP location
    def get_best_match_score(self, obj):
        # best_match(accept_lang, enabledlangs, min_score=50)
        accept_header = self.context['request'].META.get('HTTP_ACCEPT_LANGUAGE', '')
        accept_lang = []
        # Read https://docs.djangoproject.com/en/2.1/topics/i18n/translation/#internationalization-in-python-code
        for accepted, _q in parse_accept_lang_header(accept_header):
            accept_lang.append(accepted)

        # get best accepted match for this tag
        best = best_match(obj.langtag, accept_lang, min_score=50)
        # get the index of matched lang
        try:
            idx = accept_lang.index(best[0])
        except ValueError:
            return 0

        # return the score extracting the index, so the matched position at index
        # zero and score 100 will return 100. matched position at index 1 and
        # score 100 will return 99
        return best[1] - idx


class TranslationKeySerializer(serializers.ModelSerializer):
    class Meta:
        model = TranslationKey
        fields = (
            'text',
            'namespace',
        )


class TranslationKeyWithTranslationsSerializer(serializers.ModelSerializer):
    class Meta:
        model = TranslationKey
        fields = (
            'text',
            'namespace',
            'translations',
        )


class TranslationSerializer(serializers.ModelSerializer):
    translation = serializers.SerializerMethodField()
    key = TranslationKeySerializer()
    bcp47 = BCP47Serializer()

    class Meta:
        model = Translation
        fields = (
            'id',
            'key',
            'bcp47',
            'is_largue',
            'small',
            'largue',
            'translation',
        )

    def get_translation(self, obj):
        return obj.str_translation()


class BestlangtagSerializer(serializers.Serializer):
    langtag = serializers.CharField()
    accept_lang = serializers.CharField()
    score = serializers.CharField()

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass



# Generated by Django 2.0.4 on 2018-05-15 19:59

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BCP47',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('langtag', models.CharField(max_length=255, verbose_name='IETF BCP 47 langtag')),
            ],
        ),
        migrations.CreateModel(
            name='Translation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_largue', models.BooleanField(default=False)),
                ('small', models.CharField(blank=True, max_length=255, null=True, verbose_name='small')),
                ('largue', models.TextField(blank=True, null=True, verbose_name='largue')),
                ('bcp47', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='tradukoj.BCP47')),
            ],
        ),
        migrations.CreateModel(
            name='TranslationKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255, verbose_name='text key')),
            ],
        ),
        migrations.AddField(
            model_name='translation',
            name='key',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='tradukoj.TranslationKey'),
        ),
    ]



import polib
from django.core.management.base import BaseCommand
from tradukoj.models import TranslationKey


class Command(BaseCommand):
    # pylint: disable=C0301
    help = 'Remove those translations key in DB that does not exists in a given pofile'
    exitcode = 0
    data = None

    def add_arguments(self, parser):
        parser.add_argument(
            '--pofile',
            dest='pofile',
            required=True,
            help='The pofile',
        )

        parser.add_argument(
            '--namespace',
            dest='namespace',
            required=True,
            help='The namespace',
        )

        parser.add_argument(
            '--safe',
            dest='safe',
            required=False,
            help=
            'Show those keys that should be deleted but do not commit delete command on db',
            action="store_true",
        )

    def handle(self, *args, **options):
        po_file = polib.pofile(options['pofile'])
        safe_advisory = ""

        if options["safe"]:
            safe_advisory = "(Not really because --safe arg)"

        for translationkey in TranslationKey.objects.filter(
                namespace__text=options['namespace']):
            if not po_file.find(translationkey.text):
                self.stdout.write(
                    f"Delete {safe_advisory} {translationkey.text}")
                if not options["safe"]:
                    translationkey.delete()

        self.stdout.write(self.style.SUCCESS('DONE'))



# Generated by Django 2.0.4 on 2018-12-13 13:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0010_auto_20181213_1245'),
    ]

    operations = [
        migrations.AlterField(
            model_name='translationkey',
            name='namespace',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translation_keys', to='tradukoj.Namespace'),
        ),
        migrations.AlterUniqueTogether(
            name='translationkey',
            unique_together={('namespace', 'text')},
        ),
    ]



# Generated by Django 2.0.4 on 2018-12-19 18:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0011_auto_20181213_1322'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gettextfile',
            name='log',
            field=models.TextField(blank=True, max_length=1024, null=True, verbose_name='Log'),
        ),
        migrations.AlterUniqueTogether(
            name='translation',
            unique_together={('key', 'bcp47')},
        ),
    ]



from django.contrib import admin
from tradukoj import models


class TranslationKeyAdmin(admin.ModelAdmin):
    search_fields = ('text',)

class TranslationAdmin(admin.ModelAdmin):
    search_fields = ('key__text',)


admin.site.register(models.TranslationKey, TranslationKeyAdmin)
admin.site.register(models.GetTextFile)
admin.site.register(models.Namespace)
admin.site.register(models.Translation, TranslationAdmin)
admin.site.register(models.BCP47)



"""
Models definition for IETF's BCP 47 standard.

Used standard: IETF's BCP 47.
Standar RFC: http://www.rfc-editor.org/rfc/rfc5646.txt

For fun links:
https://www.w3.org/International/articles/language-tags/
https://en.wikipedia.org/wiki/IETF_language_tag


from RFC, a langtag is composed by:
langtag       = language
                ["-" script]
                ["-" region]
                *("-" variant)
                *("-" extension)
                ["-" privateuse]
So, there is:
language-extlang-script-region-variant-extension-privateuse
"""
from tempfile import NamedTemporaryFile
from django.db import models
from django.core.cache import cache
from django.conf import settings

import polib

DEFAULT_LANGUAGE_TRADUKOJ = getattr(settings, 'DEFAULT_LANGUAGE_TRADUKOJ')

class BCP47(models.Model):
    DIRECTION_LTR = 0
    DIRECTION_RTL = 1
    DIRECTION_CHOICES = ((DIRECTION_LTR, 'Left to Right (LTR)'),
                         (DIRECTION_RTL, 'Right to Left (RTL)'))

    langtag = models.CharField(
        max_length=255,
        null=False,
        blank=False,
        verbose_name='IETF BCP 47 langtag')
    name = models.CharField(
        max_length=255, null=True, blank=True, verbose_name='Lang name')
    enabled = models.BooleanField(default=True)
    # used as fallback
    default = models.BooleanField(default=False)
    direction = models.IntegerField(default=0, choices=DIRECTION_CHOICES)

    def __str__(self):
        return f"{self.langtag} - {self.name}"


class Namespace(models.Model):
    text = models.CharField(
        unique=True,
        max_length=255,
        null=False,
        blank=False,
        verbose_name='text key')

    def __str__(self):
        return self.text


class TranslationKey(models.Model):

    init_namespace = None
    auto_key_text = True

    text = models.CharField(
        max_length=255, null=False, blank=False, verbose_name='text key')

    namespace = models.ForeignKey(
        'Namespace',
        null=False,
        blank=False,
        related_name='translation_keys',
        on_delete=models.CASCADE)
    public = models.BooleanField(default=True)

    def __init__(self, *args, **kwargs):
        # create namespace into db on init_namespace set
        if 'init_namespace' in kwargs:
            self.init_namespace = kwargs['init_namespace']
            namespace, _created = Namespace.objects.get_or_create(
                text=kwargs['init_namespace'])

            kwargs['namespace'] = namespace
            del kwargs['init_namespace']

        # parameter to disable auto key text generation
        if 'auto_key_text' in kwargs:
            self.auto_key_text = kwargs['auto_key_text']
            del kwargs['auto_key_text']

        super().__init__(*args, **kwargs)

    def get_init_namespace(self):
        return self.init_namespace

    def translate(self, langtag, translation_text):
        bcp47, created = BCP47.objects.get_or_create(langtag=langtag)
        if created:
            bcp47.enabled = True
            bcp47.save()
        translation, _created = Translation.objects.get_or_create(
            key=self, bcp47=bcp47)
        if len(translation_text) > 255:
            translation.is_largue = True
            translation.largue = translation_text
        else:
            translation.is_largue = False
            translation.small = translation_text

        translation.save()

    def get_translation(self, langtag):
        try:
            translation = self.translations.get(bcp47__langtag=langtag)
            return translation
        except Translation.DoesNotExist:
            return None

    def get_other_translation(self, langtag):
        try:
            translation = self.translations.all().exclude(bcp47__langtag=langtag).first()
            return translation
        except Translation.DoesNotExist:
            return None

    def __str__(self):
        translation = self.get_translation(DEFAULT_LANGUAGE_TRADUKOJ)
        if not translation:
            translation = self.get_other_translation(DEFAULT_LANGUAGE_TRADUKOJ)
        return str(translation if translation else f"{self.namespace} - {self.text}")

    class Meta:
        unique_together = (
            'namespace',
            'text',
        )


# TODO: update cache on save event
class Translation(models.Model):
    key = models.ForeignKey(
        TranslationKey,
        null=False,
        blank=False,
        related_name='translations',
        on_delete=models.CASCADE)
    bcp47 = models.ForeignKey(
        BCP47,
        null=False,
        blank=False,
        related_name='translations',
        on_delete=models.CASCADE)
    is_largue = models.BooleanField(default=False)
    small = models.CharField(
        max_length=255, null=True, blank=True, verbose_name='small')
    largue = models.TextField(null=True, blank=True, verbose_name='largue')

    class Meta:
        unique_together = (
            'key',
            'bcp47',
        )

    def str_translation(self):
        if self.is_largue:
            return self.largue
        return self.small

    @staticmethod
    def get_cached_public_translations(langtag, namespace):
        """
        Return a big cached json of all translations.

        This will update cache if needed.

        """

        data = cache.get('tradukoj_pub_tr_{0}_{1}'.format(langtag, namespace))
        if data is None:
            Translation.update_langtag_cache(langtag, namespace)
        return cache.get('tradukoj_pub_tr_{0}_{1}'.format(langtag, namespace))

    @staticmethod
    def get_cached_private_translations(langtag, namespace):
        """
        Return a big cached json of all translations.

        This will update cache if needed.

        """
        data = cache.get('tradukoj_priv_tr_{0}_{1}'.format(langtag, namespace))
        if data is None:
            Translation.update_langtag_cache(langtag, namespace, False)
        return cache.get('tradukoj_priv_tr_{0}_{1}'.format(langtag, namespace))

    @staticmethod
    def update_langtag_cache(langtag, namespace, public=True):
        """Update cache with big json of all translations."""
        data = {langtag: {namespace: {}}}
        queryset = Translation.objects.filter(
            key__namespace__text=namespace,
            bcp47__enabled=True,
            bcp47__langtag=langtag)

        cache_string = 'tradukoj_priv_tr_{0}_{1}'.format(langtag, namespace)
        if public:
            cache_string = 'tradukoj_pub_tr_{0}_{1}'.format(langtag, namespace)
            queryset = queryset.filter(key__public=True)

        # hacemos foreach por las traducciones
        for translation in queryset:
            if not translation.str_translation():
                continue

            if not translation.str_translation().strip():
                continue

            # si la key de traducción es de tipo login.form.username
            # hacemos split en el punto (.) e iteramos sobre él.
            #
            # _current node contendrá (por referencia) el último nodo creado
            # de este mode, de un string tipo login.form.username vamos creando:
            # data[langtag][namespace]['login'] = {}
            # data[langtag][namespace]['login']['form'] = {}
            # data[langtag][namespace]['login']['form']['username'] = 'traduccion'
            deep = translation.key.text.split('.')
            _current_node = data[langtag][namespace]
            for i, element in enumerate(deep):
                # en el último elemento dejamos de profundizar y colocamos la traducción
                if i == (len(deep) - 1):
                    # Incongruity key error.
                    # _current_node debe ser un dict,
                    # de lo contrario algo ha ido mal.
                    # Posibles causas:
                    # 1) Existe una key node1.node2 = text y otra
                    #    node1.node2.node3 = text
                    #    con lo cual el algoritmo espera que node1.node2
                    #    sea un dict para añadirle una nueva clave, pero
                    #    se encuentra un string
                    if not isinstance(_current_node, dict):
                        break
                    _current_node[element] = translation.str_translation()
                    break
                if not element in _current_node:
                    _current_node[element] = {}

                # Solución parcial para gestionar
                # el fallo Incongruity key (arriba)
                # Si encuentra un elemento que se espera sea dict y no lo es,
                # sobreescribir como dict.
                # Esto provocará que node1.text2 = "text" se sobreescriba con
                # node1.text2 = {} para poder profundizar en el árbol
                # TODO: buscar cómo gestionar esto en Tradukoj
                # Opción 1: evitar keys node1.node2 si existe node1.node2.node3
                #           mediante validación en modelo
                # Opción 2: devolver un array [{traducciones}, {traducciones}]
                #           donde en el primer dict devolver las traduccionnes
                #           normales node1.node2.node3 y en el segundo dict
                #            devolver node1.node2
                if not isinstance(_current_node[element], dict):
                    _current_node[element] = {}

                # aquí está la magia, la variable _current_node es seteada por referencia
                _current_node = _current_node[element]

        if public:
            return cache.set(cache_string, data)

        return cache.set(cache_string, data)

    def __str__(self):
        return str(self.str_translation())


class GetTextFile(models.Model):
    FILE_TYPE_PO = 0
    FILE_TYPE_MO = 1
    FILE_TYPE_CHOICES = ((FILE_TYPE_PO, 'PO file'), (FILE_TYPE_MO, 'MO file'))

    file = models.FileField(
        upload_to='uploads/tradukoj'
    )
    file_type = models.IntegerField(default=0, choices=FILE_TYPE_CHOICES)
    bcp47 = models.ForeignKey(
        BCP47,
        null=False,
        blank=False,
        related_name='get_text_files',
        on_delete=models.CASCADE,
        verbose_name='Lang')
    namespace = models.ForeignKey(
        Namespace,
        null=False,
        blank=False,
        related_name='get_text_files',
        on_delete=models.CASCADE)
    last_updated_date = models.DateTimeField(
        db_index=True, auto_now=True, verbose_name='Last updated date')
    done = models.BooleanField(default=False)
    done_with_errors = models.BooleanField(default=False)
    log = models.TextField(
        null=True,
        blank=True,
        max_length=1024,
        verbose_name='Log',
    )

    def process_file(self):
        with NamedTemporaryFile() as tmpfile:
            po_file_content = self.file.open().readlines()
            tmpfile.writelines(po_file_content)
            tmpfile.seek(0)

            gettext_file = polib.pofile(tmpfile.name)
            for entry in gettext_file:
                translation_key, _created = TranslationKey.objects.get_or_create(
                    text=entry.msgid, namespace=self.namespace)

                translation, _created = Translation.objects.get_or_create(
                    key=translation_key, bcp47=self.bcp47)

                if len(entry.msgstr) > 255:
                    translation.is_largue = True
                    translation.largue = entry.msgstr
                    translation.small = None
                else:
                    translation.is_largue = False
                    translation.small = entry.msgstr
                    translation.largue = None

                translation.save()
            tmpfile.close()



    def __str__(self):
        return "{0} {1} {2} ".format(self.bcp47, self.namespace.text, self.file)



# Generated by Django 2.0.4 on 2018-11-09 20:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0005_auto_20180518_1041'),
    ]

    operations = [
        migrations.AddField(
            model_name='bcp47',
            name='direction',
            field=models.IntegerField(choices=[(0, 'Left to Right (LTR)'), (1, 'Right to Left (RTL)')], default=0),
        ),
    ]
